"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isContextTagCall = isContextTagCall;
exports.isContextFnCall = isContextFnCall;
exports.isValidFnCallContext = isValidFnCallContext;
exports.isValidTagContext = isValidTagContext;

var t = _interopRequireWildcard(require("@babel/types"));

var _utils = require("./utils");

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

var NAME = 'context';

function isContextTagCall(node, context) {
  return t.isTaggedTemplateExpression(node) && t.isMemberExpression(node.tag) && t.isCallExpression(node.tag.object) && t.isIdentifier(node.tag.object.callee) && context.hasAliasForFunc(NAME, node.tag.object.callee.name);
}

function isContextFnCall(node, context) {
  return t.isCallExpression(node) && t.isMemberExpression(node.callee) && t.isCallExpression(node.callee.object) && t.isIdentifier(node.callee.object.callee) && context.hasAliasForFunc(NAME, node.callee.object.callee.name);
}

function isValidFnCallContext(nodePath) {
  var node = nodePath.node;
  var argsLength = node.callee.object.arguments.length;

  if (argsLength !== 1) {
    throw nodePath.buildCodeFrameError("Context function accepts only 1 argument but has ".concat(argsLength, " instead."));
  }

  var contextStr = node.callee.object.arguments[0];

  if (!t.isLiteral(contextStr)) {
    throw nodePath.buildCodeFrameError("Expected string as a context argument. Actual - \"".concat((0, _utils.ast2Str)(contextStr), "\"."));
  }

  return true;
}

function isValidTagContext(nodePath) {
  var node = nodePath.node;
  var argsLength = node.tag.object.arguments.length;

  if (argsLength !== 1) {
    throw nodePath.buildCodeFrameError("Context function accepts only 1 argument but has ".concat(argsLength, " instead."));
  }

  var contextStr = node.tag.object.arguments[0];

  if (!t.isLiteral(contextStr)) {
    throw nodePath.buildCodeFrameError("Expected string as a context argument. Actual - \"".concat((0, _utils.ast2Str)(contextStr), "\"."));
  }

  return true;
}