"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.isStarted = isStarted;
exports.default = _default;

var bt = _interopRequireWildcard(require("@babel/types"));

var _fs = _interopRequireDefault(require("fs"));

var _mkdirp = _interopRequireDefault(require("mkdirp"));

var _path = _interopRequireDefault(require("path"));

var _defaults = require("./defaults");

var _poHelpers = require("./po-helpers");

var _extract = require("./extract");

var _utils = require("./utils");

var _resolve = require("./resolve");

var _errors = require("./errors");

var _context = _interopRequireDefault(require("./context"));

var _gettextContext = require("./gettext-context");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) { var desc = Object.defineProperty && Object.getOwnPropertyDescriptor ? Object.getOwnPropertyDescriptor(obj, key) : {}; if (desc.get || desc.set) { Object.defineProperty(newObj, key, desc); } else { newObj[key] = obj[key]; } } } } newObj.default = obj; return newObj; } }

function _slicedToArray(arr, i) { return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest(); }

function _nonIterableRest() { throw new TypeError("Invalid attempt to destructure non-iterable instance"); }

function _iterableToArrayLimit(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"] != null) _i["return"](); } finally { if (_d) throw _e; } } return _arr; }

function _arrayWithHoles(arr) { if (Array.isArray(arr)) return arr; }

var started = false;

function isStarted() {
  return started;
}

function _default() {
  var context;
  var disabledScopes = new Set();
  var potEntries = [];

  function tryMatchTag(cb) {
    return function (nodePath, state) {
      var node = nodePath.node;

      if ((0, _gettextContext.isContextTagCall)(node, context) && (0, _gettextContext.isValidTagContext)(nodePath)) {
        nodePath._C3PO_GETTEXT_CONTEXT = node.tag.object.arguments[0].value;
        nodePath._ORIGINAL_NODE = node;
        nodePath.node = bt.taggedTemplateExpression(node.tag.property, node.quasi);
        nodePath.node.loc = node.loc;
      }

      cb(nodePath, state);
    };
  }

  function tryMatchCall(cb) {
    return function (nodePath, state) {
      var node = nodePath.node;

      if ((0, _gettextContext.isContextFnCall)(node, context) && (0, _gettextContext.isValidFnCallContext)(nodePath)) {
        nodePath._C3PO_GETTEXT_CONTEXT = node.callee.object.arguments[0].value;
        nodePath._ORIGINAL_NODE = node;
        nodePath.node = bt.callExpression(node.callee.property, node.arguments);
        nodePath.node.loc = node.loc;
      }

      cb(nodePath, state);
    };
  }

  function extractOrResolve(nodePath, state) {
    if (nodePath.node._C3PO_visited) {
      // Should visit each node only once
      return;
    }

    if ((0, _utils.isInDisabledScope)(nodePath, disabledScopes)) {
      return;
    }

    var extractor = (0, _extract.getExtractor)(nodePath, context);

    if (!extractor) {
      return;
    }

    var aliases = context.getAliasesForFunc(extractor.name);
    var hasImport = aliases.find(context.hasImport);

    if (!hasImport // can be used in scope of context without import
    && !nodePath._C3PO_GETTEXT_CONTEXT) {
      return;
    }

    try {
      try {
        extractor.validate(nodePath.node, context);
      } catch (err) {
        if (err instanceof _errors.ValidationError) {
          context.validationFailureAction(extractor.name, err.message);
          return;
        }

        throw err;
      }

      if (context.isExtractMode()) {
        var poEntry = (0, _extract.extractPoEntry)(extractor, nodePath, context, state);
        poEntry && potEntries.push(poEntry);
        nodePath.node._C3PO_visited = true;
      }

      if (context.isResolveMode()) {
        (0, _resolve.resolveEntries)(extractor, nodePath, context, state);
        nodePath.node._C3PO_visited = true;
      }
    } catch (err) {
      // TODO: handle specific instances of errors
      throw nodePath.buildCodeFrameError("".concat(err.message, "\n").concat(err.stack));
    }
  }

  return {
    post: function post() {
      if (context && context.isExtractMode() && potEntries.length) {
        var poData = (0, _poHelpers.buildPotData)(potEntries); // Here we sort reference entries, this could be useful
        // with conf. options extract.location: 'file' and sortByMsgid
        // which simplifies merge of .po files from different
        // branches of SCM such as git or mercurial.

        var ctxs = Object.keys(poData.translations);

        var _loop = function _loop() {
          var ctx = ctxs[_i];
          var poEntries = poData.translations[ctx];
          Object.keys(poEntries).forEach(function (k) {
            var poEntry = poEntries[k]; // poEntry has a form:
            // {
            //     msgid: 'message identifier',
            //     msgstr: 'translation string',
            //     comments: {
            //         reference: 'path/to/file.js:line_number\npath/to/other/file.js:line_number'
            //     }
            // }

            if (poEntry.comments && poEntry.comments.reference) {
              poEntry.comments.reference = poEntry.comments.reference.split('\n').sort(_utils.poReferenceComparator).join('\n');
            }
          });

          if (context.isSortedByMsgid()) {
            var oldPoData = poData.translations[ctx];
            var newContext = {};
            var keys = Object.keys(oldPoData).sort();
            keys.forEach(function (k) {
              newContext[k] = oldPoData[k];
            });
            poData.translations[ctx] = newContext;
          }
        };

        for (var _i = 0; _i < ctxs.length; _i++) {
          _loop();
        }

        var potStr = (0, _poHelpers.makePotStr)(poData);
        var filepath = context.getOutputFilepath();

        var dirPath = _path.default.dirname(filepath);

        _mkdirp.default.sync(dirPath);

        _fs.default.writeFileSync(filepath, potStr);
      }
    },
    visitor: {
      TaggedTemplateExpression: tryMatchTag(extractOrResolve),
      CallExpression: tryMatchCall(extractOrResolve),
      Program: function Program(nodePath, state) {
        started = true;

        if (!context) {
          context = new _context.default(state.opts);
        } else {
          context.clear();
        }

        disabledScopes = new Set();

        if ((0, _utils.hasDisablingComment)(nodePath.node)) {
          disabledScopes.add(nodePath.scope.uid);
        }
      },
      BlockStatement: function BlockStatement(nodePath) {
        if ((0, _utils.hasDisablingComment)(nodePath.node)) {
          disabledScopes.add(nodePath.scope.uid);
        }
      },
      VariableDeclarator: function VariableDeclarator(nodePath) {
        var node = nodePath.node;
        if (!(0, _utils.isTtagRequire)(node)) return; // require calls

        node.id.properties.map(function (_ref) {
          var keyName = _ref.key.name,
              valueName = _ref.value.name;
          return [keyName, valueName];
        }).filter(function (_ref2) {
          var _ref3 = _slicedToArray(_ref2, 1),
              keyName = _ref3[0];

          return _defaults.ALIAS_TO_FUNC_MAP[keyName];
        }).forEach(function (_ref4) {
          var _ref5 = _slicedToArray(_ref4, 2),
              keyName = _ref5[0],
              valueName = _ref5[1];

          if (keyName !== valueName) {
            // if alias
            context.addAlias(_defaults.ALIAS_TO_FUNC_MAP[keyName], valueName);
            context.addImport(valueName);
          } else {
            context.addImport(keyName);
          }
        });
      },
      ImportDeclaration: function ImportDeclaration(nodePath, state) {
        var node = nodePath.node;

        if (!context) {
          context = new _context.default(state.opts);
        }

        if ((0, _utils.isTtagImport)(node)) {
          node.specifiers.filter(function (_ref6) {
            var name = _ref6.local.name;
            return _defaults.ALIAS_TO_FUNC_MAP[name];
          }).forEach(function (s) {
            return context.addImport(s.local.name);
          });
        }

        if ((0, _utils.isTtagImport)(node) && (0, _utils.hasImportSpecifier)(node)) {
          node.specifiers.filter(bt.isImportSpecifier).filter(function (_ref7) {
            var name = _ref7.imported.name;
            return _defaults.ALIAS_TO_FUNC_MAP[name];
          }).forEach(function (_ref8) {
            var imported = _ref8.imported,
                local = _ref8.local;
            context.addAlias(_defaults.ALIAS_TO_FUNC_MAP[imported.name], local.name);
            context.addImport(local.name);
          });
        }
      }
    }
  };
}